package jp.mehve.httpproxy;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import compat.com.sun.net.httpserver.Headers;
import compat.com.sun.net.httpserver.HttpExchange;
import compat.com.sun.net.httpserver.HttpHandler;
import compat.sun.net.httpserver.HttpExchangeHack;

public class ProxyHandler implements HttpHandler {
    private static final String HEADER_KEY_CONTENT_LENGTH = "Content-length"; //$NON-NLS-1$
    
    public ProxyHandler() {
        // DO_NOTHING
    }
    
    private void handleRequest(HttpExchange t, HttpURLConnection con) throws IOException {
        con.setRequestMethod(t.getRequestMethod());
        
        // --- request header
        for (Entry<String, List<String>> header : t.getRequestHeaders().entrySet()) {
            String key = header.getKey();
            if (key == null) {
                continue;
            }
            if (HEADER_KEY_CONTENT_LENGTH.equalsIgnoreCase(key)) {
                continue;
            }
            for (String value : header.getValue()) {
                con.setRequestProperty(key, value);
            }
        }

        // --- Request Body
        {
            InputStream is = t.getRequestBody();
            ByteArrayOutputStream os = new ByteArrayOutputStream();
            byte[] buf = new byte[65536];
            int len = 0;
            while ((len = is.read(buf)) > 0) {
                os.write(buf, 0, len);
                HttpExchangeHack.hack(t);
            }

            if (os.size() > 0) {
                byte[] barray = os.toByteArray();
                con.setRequestProperty(HEADER_KEY_CONTENT_LENGTH, String.valueOf(barray.length));
                con.getOutputStream().write(barray);
            }
        }
    }

    private void handleResponse(HttpExchange t, HttpURLConnection con) throws IOException {
        int responseCode = con.getResponseCode();
        long contentLength = -1;
        boolean chunked = false;
        boolean autosave = false;

        {
            Headers responseHeaders = t.getResponseHeaders();
            Map<String, List<String>> headerMapping = new HashMap<String, List<String>>();
            for (int i = 0; con.getHeaderFieldKey(i) != null || con.getHeaderField(i) != null; ++i) {
                String key = con.getHeaderFieldKey(i);
                String val = con.getHeaderField(i);
                if (key == null) {
                    // skip.
                    continue;
                }
                if ("Transfer-encoding".equalsIgnoreCase(key) && "chunked".equalsIgnoreCase(val)) {
                    chunked = true;
                }
                if (HEADER_KEY_CONTENT_LENGTH.equalsIgnoreCase(key)) {
                    if (contentLength < 0) {
                        contentLength = Long.parseLong(val);
                    } else {
                        // seems malicious.
                        throw new IOException("seems malicious 1");
                    }
                    // skip. Content-length will be set at t.sendResponseHeaders();
                    continue;
                }
                List<String> lst = headerMapping.containsKey(key) ? headerMapping.get(key) : new ArrayList<String>();
                lst.add(val);
                headerMapping.put(key, lst);
            }
            responseHeaders.putAll(headerMapping);
        }

        if ("TRUE".equals(t.getResponseHeaders().getFirst(ProxyHandlerConstants.ATTR_FORCED_CHUNKED))) {
            chunked = true;
            contentLength = 0;
        }

        // ----- RESPONSE BODY -----
        if (chunked) {
            // XXX: If 'chunked' used, contentLength shouldn't been set. If so, this seems malicious, therefore we
            // reject this response.
            if (contentLength > 0) { throw new IOException("Seems Malicious"); }
            contentLength = 0;
        }

        // Some case has (Content-length == 0) && !chunked.
        if (contentLength < 0 || (!chunked && contentLength == 0)) {
            t.sendResponseHeaders(responseCode, -1);
            // skip body.
        } else {
            // This case means contentLength > 0 or (contentLength == 0 && chunked)
            // chunked && contentLength > 0 seems malicious.

            t.sendResponseHeaders(responseCode, contentLength);
            InputStream is = con.getInputStream();
            OutputStream os = t.getResponseBody();
            
            FileOutputStream fos = null;
            if ("TRUE".equals(t.getResponseHeaders().getFirst(ProxyHandlerConstants.ATTR_AUTOSAVE))) {
                System.err.println("Saving... " + t.getRequestURI().toString());
                File tmpFile = File.createTempFile("smile", ".flv", new File("."));
                System.err.println("to " + tmpFile.toString());
                fos = new FileOutputStream(tmpFile);
                System.err.flush();
            }
            
            byte[] buf = new byte[65536];
            int len = 0;
            int sum = 0;
            while ((len = is.read(buf)) > 0) {
                sum += len;
                os.write(buf, 0, len);
                if (fos != null) { fos.write(buf, 0, len); }
                HttpExchangeHack.hack(t);
            }
            is.close();
            os.close();
            if (fos != null) { fos.flush(); fos.close(); }
        }
    }

    public void handle(HttpExchange t) throws IOException {
        System.out.println("BEGIN: " + t.getRequestURI().toString());
        System.out.flush();
        try {
            URL url = new URL(t.getRequestURI().toString());

            HttpURLConnection connection = (HttpURLConnection) url.openConnection();
            connection.setDoOutput(true);
            connection.setInstanceFollowRedirects(false);
            
            System.out.println("CON: " + t.getRequestURI().toString());

            handleRequest(t, connection);
            connection.connect();
            handleResponse(t, connection);
        } catch (IOException e) {
            e.printStackTrace();
            throw e;
        } finally {
            t.close();

            System.out.println("END: " + t.getRequestURI().toString());
            System.out.flush();
        }
    }
}
