package jp.mehve.io;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;

/**
 * pipedIs and os will be closed in this class. However, pipedOs should be closed by user.
 * 
 * Data written to pipedOs are transfered to os.
 * 
 * @author shinyak
 * 
 */
public class PipeThread extends Thread {
    private PipedInputStream  pipedIs;
    private PipedOutputStream pipedOs;
    private OutputStream      os;

    public PipeThread(OutputStream os) throws IOException {
        this.pipedIs = new PipedInputStream();
        this.pipedOs = new PipedOutputStream(this.pipedIs);
        this.os = os;
    }

    public PipedInputStream getPipedInputStream() {
        return this.pipedIs;
    }
    
    public PipedOutputStream getPipedOutputStream() {
        return this.pipedOs;
    }
    
    /**
     * you can override this method to modify streaming data
     * 
     * @param is
     * @param os
     * @throws IOException
     */
    protected void pipe(InputStream is, OutputStream os) throws Exception {
        byte[] buf = new byte[8192];
        int len = 0;

        while ((len = is.read(buf)) > 0) {
            os.write(buf, 0, len);
        }
    }

    @Override
    public void run() {
        // System.err.println("BEGIN");
        try {
            pipe(this.pipedIs, this.os);
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                pipedIs.close();
            } catch (IOException e) {
            } finally {
                try {
                    os.close();
                } catch (IOException e) {
                }
            }
        }
        // System.err.println("END");
    }

}
