package jp.mehve.nicofilter.filter.nicofilter;

import java.util.ArrayList;

import org.xml.sax.Attributes;
import org.xml.sax.helpers.DefaultHandler;

public class NicommentSAXHandler extends DefaultHandler {
    private static final String PACKET         = "packet";                 //$NON-NLS-1$
    private static final String VIEW_COUNTER   = "view_counter";           //$NON-NLS-1$
    private static final String CHAT           = "chat";                   //$NON-NLS-1$
    private static final String THREAD         = "thread";                 //$NON-NLS-1$

    private ArrayList<NicoChat> chats          = new ArrayList<NicoChat>();
    private Nicomment           nicomment      = null;
    private NicoViewCounter     viewCounter    = null;
    private NicoThread          thread         = null;
    private NicoChat            currentComment = null;

    public Nicomment getNicomment() {
        return this.nicomment;
    }

    @Override
    public void startDocument() {
        // Do Nothing
    }

    private void startElementChat(Attributes attributes) {
        assert (this.currentComment == null);

        String date = attributes.getValue("date"); //$NON-NLS-1$
        String mail = attributes.getValue("mail"); //$NON-NLS-1$
        String no = attributes.getValue("no"); //$NON-NLS-1$
        String premium = attributes.getValue("premium"); //$NON-NLS-1$
        String thread = attributes.getValue("thread"); //$NON-NLS-1$
        String userId = attributes.getValue("user_id"); //$NON-NLS-1$
        String vpos = attributes.getValue("vpos"); //$NON-NLS-1$
        this.currentComment = new NicoChat(date, mail, no, premium, thread, userId, vpos);
    }

    private void startElementThread(Attributes attributes) {
        // <thread last_res="3717" resultcode="0" thread="1180470813" ticket="0xb3291b0"/>
        String lastRes = attributes.getValue("last_res"); //$NON-NLS-1$
        String resultCode = attributes.getValue("resultcode"); //$NON-NLS-1$
        String thread = attributes.getValue("thread"); //$NON-NLS-1$
        String ticket = attributes.getValue("ticket"); //$NON-NLS-1$
        this.thread = new NicoThread(lastRes, resultCode, thread, ticket);
    }

    private void startElementViewCounter(Attributes attributes) {
        // <view_counter video="5398"/>
        String video = attributes.getValue("video"); //$NON-NLS-1$
        this.viewCounter = new NicoViewCounter(video);
    }

    @Override
    public void startElement(String uri, String localName, String qName, Attributes attributes) {
        if (PACKET.equals(qName)) {
            return;
        } else if (THREAD.equals(qName)) {
            startElementThread(attributes);
        } else if (VIEW_COUNTER.equals(qName)) {
            startElementViewCounter(attributes);
        } else if (CHAT.equals(qName)) {
            startElementChat(attributes);
        }
    }

    @Override
    public void characters(char[] ch, int offset, int length) {
        if (this.currentComment == null) { return; }
        this.currentComment.comment = new String(ch, offset, length);
    }

    @Override
    public void endElement(String uri, String localName, String qName) {
        if (!CHAT.equals(qName)) { return; }

        this.chats.add(this.currentComment);
        this.currentComment = null;
    }

    @Override
    public void endDocument() {
        this.nicomment = new Nicomment(this.thread, this.viewCounter, this.chats);
    }
}
