package jp.mehve.nicofilter.filter.nicofilter;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.Map;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.SAXException;

public class Reputation {
    // userid -> rating
    private Map<Integer, Integer> rating = null;
    private NicommentFilter       filter;

    public Reputation(NicommentFilter filter) {
        this.filter = filter;
    }

    public void clear() {
        this.rating = null;
    }
    
    public synchronized boolean hasRating() {
        return this.rating != null;
    }

    public synchronized int getRating(int userId) {
        if (this.rating == null) {
            this.rating = getRatingFromServer();
        }

        if (this.rating != null && this.rating.containsKey(userId)) {
            return this.rating.get(userId);
        } else {
            return 0;
        }
    }

    private Map<Integer, Integer> getRatingFromServer() {
        byte[] ratingResponseContent = getRatingResponse();
        if (ratingResponseContent == null) { return null; }

        try {
            SAXParser saxParser = SAXParserFactory.newInstance().newSAXParser();
            ReputationSAXHandler saxHandler = new ReputationSAXHandler();
            saxParser.parse(new ByteArrayInputStream(ratingResponseContent), saxHandler);
            
            return saxHandler.getReputationMapping();
        } catch (IOException e) {
            e.printStackTrace();
        } catch (ParserConfigurationException e) {
            e.printStackTrace();
        } catch (SAXException e) {
            e.printStackTrace();
        }

        return null;
    }

    private byte[] getRatingResponse() {
        String cookie = this.filter.getStolenCookie();
        String stolenThreadNo = "1182655909";
        if (cookie == null || stolenThreadNo == null) { return null; }

        try {
            String threadNo = this.filter.getStolenThreadNo();
            if (threadNo == null) { return null; }
            URL url = new URL("http://msg12.nicovideo.jp/api/");
            HttpURLConnection con = (HttpURLConnection) url.openConnection();
            con.setDoOutput(true);
            con.setInstanceFollowRedirects(false);

            String body = "<thread res_from=\"-500\" version=\"20061206\"" + " thread=\"" + stolenThreadNo + "\" />";
            byte[] ba = body.getBytes("utf-8");

            con.setRequestMethod("POST");
            con.setRequestProperty("Referer", "http://www.nicovideo.jp/");
            con.setRequestProperty("Cookie", cookie);
            con.setRequestProperty("Host", "www.nicovideo.jp");
            con.setRequestProperty("Content-length", String.valueOf(ba.length));
            con.getOutputStream().write(ba);

            con.connect();

            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            byte[] buf = new byte[8192];
            int len;
            while ((len = con.getInputStream().read(buf)) > 0) {
                baos.write(buf, 0, len);
            }
            return baos.toByteArray();

        } catch (Exception e) {
            e.printStackTrace();
        }

        return null;
    }

    public synchronized void setRating(int userId, int newRating) {
        this.rating.put(userId, newRating);
    }
}
