package jp.mehve.nicofilter.filter.nicofilter;

import java.util.HashMap;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import jp.mehve.util.Pair;

import org.xml.sax.Attributes;
import org.xml.sax.helpers.DefaultHandler;

public class ReputationSAXHandler extends DefaultHandler {
    private static final String   CHAT        = "chat";
    private boolean               isInChat    = false;
    private Pattern               goodComment = Pattern.compile("(I|i)(D|d):?\\s*(\\d+)\\s*乙");
    private Pattern               badComment  = Pattern.compile("(I|i)(D|d):?\\s*(\\d+)\\s*自重しろ");
    private Map<Integer, Integer> mapping     = new HashMap<Integer, Integer>();
    private int                   user_id     = -1;
    private Map<Pair<Integer, Integer>, Integer> incrDecrHistory =
        new HashMap<Pair<Integer, Integer>, Integer>();

    public void characters(char[] ch, int offset, int length) {
        if (!this.isInChat) { return; }

        String text = new String(ch, offset, length);
        Matcher goodMatcher = goodComment.matcher(text);
        String goodMatchedString = goodMatcher.matches() ? goodMatcher.group(3) : null;
        Matcher badMatcher = badComment.matcher(text);
        String badMatchedString = badMatcher.matches() ? badMatcher.group(3) : null;

        // TODO:
        // If written ID is the same as the moderator id,
        // this seems malicious repuration. He/She should have penalty.
        // If some ID is repurated repeatedly, the moderator seems malicious.
        // He/She also has penalty.
        if (goodMatchedString != null) {
            System.err.println("Good: " + goodMatchedString);
            incrRating(this.user_id, Integer.parseInt(goodMatchedString));
        } else if (badMatchedString != null) {
            System.err.println("Bad: " + badMatchedString);
            decrRating(this.user_id, Integer.parseInt(badMatchedString));
        }
    }

    public void endDocument() {
    }

    public void endElement(String uri, String localName, String qName) {
        if (CHAT.equals(qName)) {
            assert (this.isInChat);
            this.isInChat = false;
        }
    }

    public Map<Integer, Integer> getReputationMapping() {
        return mapping;
    }

    public void startDocument() {
        // Do Nothing
    }

    public void startElement(String uri, String localName, String qName, Attributes attributes) {
        if (CHAT.equals(qName)) {
            assert (!this.isInChat);
            this.isInChat = true;
            try {
                this.user_id = Integer.parseInt(attributes.getValue("user_id"));
            } catch (NumberFormatException e) {
                this.user_id = -1;
            }
        }
    }

    private void incrRating(int who, int whom) {
        if (who < 0 || whom < 0) { return; }
        if (who == whom) {
            decrRating(who, who);
        }
        
        incrIncrDecrHistory(who, whom);
        int v = mapping.get(whom) == null ? 0 : mapping.get(whom);
        mapping.put(whom, v + 1);
    }

    private void decrRating(int who, int whom) {
        if (who < 0 || whom < 0) { return; }
        
        incrIncrDecrHistory(who, whom);
        int v = mapping.get(whom) == null ? 0 : mapping.get(whom);
        mapping.put(whom, v + 1);
    }
    
    private void incrIncrDecrHistory(int who, int whom) {
        Pair<Integer, Integer> key = new Pair<Integer, Integer>(who, whom);
        Integer v = incrDecrHistory.get(key) == null ? 0 : incrDecrHistory.get(key);
        incrDecrHistory.put(key, v + 1);
    }
}
