package jp.mehve.nicofilter.mainui;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

// <config>
//   <nicofilter>
//     <removeNGID>true/false</removeNGID>
//     <removeNGWord>true/false</removeNGWord>
//     <reputation>true/false</reputation>
//   </nicofilter>
//   <NGWords>
//     <word></word>*
//   </NGWords>
//   <NGIDs>
//     <id></id>*
//   </NGIDs>
// </config>
//

/**
 * XXX: Bad Programming Manner!
 * 
 * XXX: create synchronized block when using these fields.
 */
public class ConfigHolder {
    public boolean       removeNGID;
    public boolean       removeNGWord;
    public boolean       reputation;
    public boolean       autosaveFLV;
    public List<String>  ngWords;
    public List<String>  ngIDs;
    public Object        mutex;

    public ConfigHolder() {
        this.removeNGID = true;
        this.removeNGWord = true;
        this.reputation = false;
        this.autosaveFLV = false;
        this.ngWords = new ArrayList<String>();
        this.ngIDs = new ArrayList<String>();
        this.mutex = new Object();
    }

    private void loadSub(String filename) throws ParserConfigurationException, SAXException, IOException {
        DocumentBuilderFactory dbfactory = DocumentBuilderFactory.newInstance();
        DocumentBuilder builder = dbfactory.newDocumentBuilder();
        Document doc = builder.parse(new File(filename));
        Element root = doc.getDocumentElement();

        // XXX: should do null-check!!!
        loadNicoFilter((Element) root.getElementsByTagName("nicofilter").item(0));
        loadNGWords((Element) root.getElementsByTagName("NGWords").item(0));
        loadNGIDs((Element) root.getElementsByTagName("NGIDs").item(0));
    }

    private void loadNGIDs(Element elem) {
        this.ngIDs.clear();

        NodeList words = elem.getElementsByTagName("id");
        for (int i = 0; i < words.getLength(); ++i) {
            Node node = words.item(i);
            if (!(node instanceof Element)) {
                continue;
            }
            Element elm = (Element) node;
            if (elm.getTagName().equals("id")) {
                try {
                    this.ngIDs.add(elm.getTextContent());
                } catch (NumberFormatException e) {
                    // do nothing
                }
            }
        }
    }

    private void loadNGWords(Element elem) {
        this.ngWords.clear();

        NodeList words = elem.getElementsByTagName("word");
        for (int i = 0; i < words.getLength(); ++i) {
            Node node = words.item(i);
            if (!(node instanceof Element)) {
                continue;
            }
            Element elm = (Element) node;
            if (elm.getTagName().equals("word")) {
                this.ngWords.add(elm.getTextContent());
            }
        }
    }

    private boolean loadNicoFilter(Element elem) {
        if (elem == null) { return false; }
        Element removeNGID = (Element) elem.getElementsByTagName("removeNGID").item(0);
        Element removeNGWord = (Element) elem.getElementsByTagName("removeNGWord").item(0);
        Element reputation = (Element) elem.getElementsByTagName("reputation").item(0);
        Element autosave = (Element) elem.getElementsByTagName("autosave").item(0);

        if (removeNGID != null) {
            String str = removeNGID.getTextContent();
            if (str != null) {
                this.removeNGID = Boolean.parseBoolean(str);
            }
        }

        if (removeNGWord != null) {
            String str = removeNGWord.getTextContent();
            if (str != null) {
                this.removeNGWord = Boolean.parseBoolean(str);
            }
        }

        if (reputation != null) {
            String str = reputation.getTextContent();
            if (str != null) {
                this.reputation = Boolean.parseBoolean(str);
            }
        }
        
        if (autosave != null) {
            String str = autosave.getTextContent();
            if (str != null) {
                this.autosaveFLV = Boolean.parseBoolean(str);
            }
        }

        return true;
    }

    public boolean load(String filename) {
        try {
            loadSub(filename);
            return true;
        } catch (ParserConfigurationException e) {
            return false;
        } catch (SAXException e) {
            return false;
        } catch (IOException e) {
            return false;
        }
    }

    private void saveSub(String filename) throws FileNotFoundException, UnsupportedEncodingException, IOException {
        FileOutputStream fos = new FileOutputStream(filename);
        PrintStream out = new PrintStream(fos, true, "utf-8");

        out.append("<?xml version=\"1.0\" encoding=\"utf-8\"?>\n");
        out.append("<config>\n");
        out.append("<nicofilter>\n");
        out.append("  <removeNGID>").append(Boolean.toString(this.removeNGID)).append("</removeNGID>\n");
        out.append("  <removeNGWord>").append(Boolean.toString(this.removeNGWord)).append("</removeNGWord>\n");
        out.append("  <reputation>").append(Boolean.toString(this.reputation)).append("</reputation>\n");
        out.append("  <autosave>").append(Boolean.toString(this.autosaveFLV)).append("</autosave>\n");
        out.append("</nicofilter>\n");
        out.append("<NGWords>\n");
        for (String w : this.ngWords) {
            out.append("  <word>").append(w).append("</word>\n");
        }
        out.append("</NGWords>\n");
        out.append("<NGIDs>\n");
        for (String i : this.ngIDs) {
            out.append("  <id>").append(i).append("</id>\n");
        }
        out.append("</NGIDs>\n");
        out.append("</config>\n");

        out.flush();
        out.close();
        fos.close();
    }

    public boolean save(String filename) {
        try {
            saveSub(filename);
            return true;
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }

        return false;
    }
}
