package jp.mehve.nicofilter.mainui;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableColumn;

import jp.mehve.nicofilter.filter.nicofilter.NicoChat;
import jp.mehve.nicofilter.filter.nicofilter.NicommentFilter;

public class InformationPanel extends JPanel implements ItemListener, ActionListener {
    private static String     AC_START_PROXY = "StartProxy"; //$NON-NLS-1$
    private static String     AC_STOP_PROXY  = "StopProxy"; //$NON-NLS-1$
    private static String     AC_RELOAD      = "Reload";  //$NON-NLS-1$
    private static String     AC_AUTOSAVEFLV = "AutoSaveFLV"; 
    
    private NicommentFilter   filter;
    private ConfigHolder      config;

    private JLabel            proxyStatus;
    private JLabel            autosaveStatus;
    private JCheckBox         ngIdCheckBox;
    private JCheckBox         ngWordCheckBox;
    private JCheckBox         useReputationCheckBox;
    private JLabel            threadLabel;
    private JTable            commentTable;
    private DefaultTableModel tableModel;
    
    private JButton           startButton;
    private JButton           stopButton;
    private JButton           reloadButton;
    private JButton           autoSaveFLVButton;

    public InformationPanel(NicommentFilter filter, ConfigHolder config) {
        this.filter = filter;
        this.config = config;
        createGUI();

        if (filter != null) {
            filter.setPanel(this);
        }
    }
    
    public void addComment(NicoChat chat) {
        StringBuilder sb = new StringBuilder();
        if ("1".equals(chat.premium)) { sb.append("P"); }
        if (chat.removed) { sb.append("R"); }
        
        tableModel.insertRow(0, new String[] { chat.userId, sb.toString(), chat.mail, chat.comment, });
        // TODO: Magic Number! should be changable.
        if (tableModel.getRowCount() > 500) {
            tableModel.removeRow(500);
        }   
    }

    public void itemStateChanged(ItemEvent e) {
        Object obj = e.getSource();

        synchronized (config.mutex) {
            if (obj == ngIdCheckBox) {
                config.removeNGID = ngIdCheckBox.isSelected();
            } else if (obj == ngWordCheckBox) {
                config.removeNGWord = ngWordCheckBox.isSelected();
            } else if (obj == useReputationCheckBox) {
                config.reputation = useReputationCheckBox.isSelected();
            } else {
                // DO_NOTHING
            }
        }
    }

    private void createGUI() {
        proxyStatus = new JLabel("Proxy: ");
        autosaveStatus = new JLabel("Autosave: ");
        ngIdCheckBox = new JCheckBox("Filter NG ID");
        ngWordCheckBox = new JCheckBox("Filter NG Word");
        threadLabel = new JLabel("Latest Thread: ");
        useReputationCheckBox = new JCheckBox("Reputation");
        tableModel = new DefaultTableModel(new String[] { "ID", "MARKS", "COMMAND", "COMMENT" }, 0);
        commentTable = new JTable(tableModel);
        JScrollPane scrollPane = new JScrollPane(commentTable);

        // TODO: images should be placed in resource folder.
        
        startButton = new JButton(new ImageIcon(getClass().getResource("img/start.png")));
        stopButton = new JButton(new ImageIcon(getClass().getResource("img/stop.png")));
        reloadButton = new JButton(new ImageIcon(getClass().getResource("img/reload.png")));
        autoSaveFLVButton = new JButton(new ImageIcon(getClass().getResource("img/autosave.png")));
        
        GridBagLayout gridBag = new GridBagLayout();
        GridBagConstraints constraints = new GridBagConstraints();
        constraints.fill = GridBagConstraints.BOTH;
        constraints.weightx = 20.0;
        constraints.weighty = 100.0;

        setBorder(BorderFactory.createEmptyBorder(3, 3, 3, 3)); // padding
        setLayout(gridBag);

        {
            JPanel buttonPanel = new JPanel();
            this.startButton.setMaximumSize(new Dimension(32, 32));
            this.stopButton.setMaximumSize(new Dimension(32, 32));
            this.reloadButton.setMaximumSize(new Dimension(32, 32));
            this.autoSaveFLVButton.setMaximumSize(new Dimension(32, 32));
            this.startButton.setMinimumSize(new Dimension(32, 32));
            this.stopButton.setMinimumSize(new Dimension(32, 32));
            this.reloadButton.setMinimumSize(new Dimension(32, 32));
            this.autoSaveFLVButton.setMinimumSize(new Dimension(32, 32));
            this.startButton.setPreferredSize(new Dimension(32, 32));
            this.stopButton.setPreferredSize(new Dimension(32, 32));
            this.reloadButton.setPreferredSize(new Dimension(32, 32));
            this.autoSaveFLVButton.setPreferredSize(new Dimension(32, 32));

            buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.LINE_AXIS));
            buttonPanel.add(Box.createRigidArea(new Dimension(5, 0)));
            buttonPanel.add(this.stopButton);
            this.stopButton.setActionCommand(AC_STOP_PROXY);
            this.stopButton.addActionListener(this);
            buttonPanel.add(Box.createRigidArea(new Dimension(5, 0)));
            buttonPanel.add(this.startButton);
            this.startButton.setActionCommand(AC_START_PROXY);
            this.startButton.addActionListener(this);
            buttonPanel.add(Box.createRigidArea(new Dimension(5, 0)));
            buttonPanel.add(this.reloadButton);
            this.reloadButton.setActionCommand(AC_RELOAD);
            this.reloadButton.addActionListener(this);
            buttonPanel.add(Box.createRigidArea(new Dimension(5, 0)));
            buttonPanel.add(this.autoSaveFLVButton);
            this.autoSaveFLVButton.setActionCommand(AC_AUTOSAVEFLV);
            this.autoSaveFLVButton.addActionListener(this);
            buttonPanel.add(Box.createGlue());

            constraints.gridx = 0;
            constraints.gridy = GridBagConstraints.RELATIVE;
            constraints.gridwidth = 1;
            constraints.gridheight = 1;
            gridBag.setConstraints(buttonPanel, constraints);
            add(buttonPanel);
        }

        {
            constraints.gridx = 0;
            constraints.gridy = GridBagConstraints.RELATIVE;
            constraints.gridwidth = GridBagConstraints.REMAINDER;
            constraints.gridheight = 1;
            gridBag.setConstraints(this.proxyStatus, constraints);
            add(this.proxyStatus);
        }

        {
            constraints.gridx = 0;
            constraints.gridy = GridBagConstraints.RELATIVE;
            constraints.gridwidth = GridBagConstraints.REMAINDER;
            constraints.gridheight = 1;
            gridBag.setConstraints(this.threadLabel, constraints);
            add(this.threadLabel);
        }
        
        {
            constraints.gridx = 0;
            constraints.gridy = GridBagConstraints.RELATIVE;
            constraints.gridwidth = GridBagConstraints.REMAINDER;
            constraints.gridheight = 1;
            gridBag.setConstraints(this.autosaveStatus, constraints);
            add(this.autosaveStatus);
        }

        {
            constraints.gridx = 0;
            constraints.gridy = GridBagConstraints.RELATIVE;
            constraints.gridwidth = 1;
            constraints.gridheight = 1;
            gridBag.setConstraints(this.useReputationCheckBox, constraints);
            add(this.useReputationCheckBox);
            this.useReputationCheckBox.addItemListener(this);
        }

        {
            constraints.gridx = 0;
            constraints.gridy = GridBagConstraints.RELATIVE;
            constraints.gridwidth = 1;
            constraints.gridheight = 1;
            gridBag.setConstraints(this.ngIdCheckBox, constraints);
            add(this.ngIdCheckBox);
            this.ngIdCheckBox.addItemListener(this);
        }

        {
            constraints.gridx = 0;
            constraints.gridy = GridBagConstraints.RELATIVE;
            constraints.gridwidth = 1;
            constraints.gridheight = 1;
            gridBag.setConstraints(ngWordCheckBox, constraints);
            add(this.ngWordCheckBox);
            this.ngWordCheckBox.addItemListener(this);
        }

        {
            constraints.gridx = 1;
            constraints.gridy = 0;
            constraints.gridwidth = 1;
            constraints.gridheight = GridBagConstraints.REMAINDER;
            constraints.weightx = 80.0;
            this.commentTable.getTableHeader().setReorderingAllowed(false);
            this.commentTable.setDefaultEditor(Object.class, null);
            TableColumn idColumn = this.commentTable.getColumn("ID");
            idColumn.setPreferredWidth(70);
            idColumn.setMaxWidth(70);
            TableColumn marksColumn = this.commentTable.getColumn("MARKS");
            marksColumn.setPreferredWidth(50);
            marksColumn.setMaxWidth(50);
            TableColumn commandColumn = this.commentTable.getColumn("COMMAND");
            commandColumn.setPreferredWidth(70);
            TableColumn commentColumn = this.commentTable.getColumn("COMMENT");
            commentColumn.setPreferredWidth(220);
            gridBag.setConstraints(scrollPane, constraints);
            add(scrollPane);
        }
    }

    @Override
    protected void paintComponent(Graphics g) {
        if (Main.isProxyRunning()) {
            this.startButton.setEnabled(false);
            this.stopButton.setEnabled(true);
            proxyStatus.setText("Proxy: Running");
        } else {
            this.startButton.setEnabled(true);
            this.stopButton.setEnabled(false);
            proxyStatus.setText("Proxy: Stop");
        }
        
        synchronized (config.mutex) {
            if (config.autosaveFLV) {
                autosaveStatus.setText("Autosave: Yes");
            } else {
                autosaveStatus.setText("Autosave: No");
            }
        }

        if (filter != null) {
            synchronized (config.mutex) {
                ngIdCheckBox.setSelected(config.removeNGID);
                ngWordCheckBox.setSelected(config.removeNGWord);
                useReputationCheckBox.setSelected(config.reputation);
            }

            if (this.filter.getStolenThreadNo() != null) {
                threadLabel.setText("Thread: " + filter.getStolenThreadNo());
            }
        }

        super.paintComponent(g);
    }

    public void actionPerformed(ActionEvent ae) {
        String cmd = ae.getActionCommand();
        if (AC_START_PROXY.equals(cmd)) {
            Main.startProxy();
        } else if (AC_STOP_PROXY.equals(cmd)) {
            Main.stopProxy();
        } else if (AC_RELOAD.equals(cmd)) {
            filter.clearReputation();
            repaint();
        } else if (AC_AUTOSAVEFLV.equals(cmd)) {
            synchronized (config.mutex) {
                config.autosaveFLV = !config.autosaveFLV;
            }
            repaint();
        }
    }
}
