package jp.mehve.util;

class TrieMapTree<T> {
    private static final int BRANCHES = 26;
    private TrieMapTree<T>[] next;
    private T elem;
    
    public TrieMapTree() {
        next = new TrieMapTree[BRANCHES];
        elem = null;
    }
    
    public void add(String key, int idx, T val) {
        if (key.length() == idx) {
            elem = val;
            return;
        }
        
        int index = indexOf(key.charAt(idx));
        if (next[index] == null) {
            next[index] = new TrieMapTree<T>();
        }
        next[index].add(key, idx + 1, val);
    }
    
    public T get(String key, int idx) {
        if (key.length() == idx) {
            return elem;
        }
        
        int index = indexOf(key.charAt(idx));
        if (next[index] != null) {
            return next[index].get(key, idx + 1);
        } else {
            return null;
        }
    }
    
    private int indexOf(char c) {
        if ('a' <= c && c <= 'z') { return c - 'a'; }
        if ('A' <= c && c <= 'Z') { return c - 'A'; }
        
        throw new IllegalArgumentException("Invalid : " + c);
    }
}

public class TrieMap<T> {
    private TrieMapTree<T> tree;
    
    public TrieMap() {
        this.tree = new TrieMapTree<T>();
    }
    
    public void add(String key, T val) {
        tree.add(key, 0, val);
    }
    
    public T get(String key) {
        return tree.get(key, 0);
    }
    
    public boolean contains(String key) {
        return tree.get(key, 0) != null;
    }
}


