package jp.mehve.util;

class TrieSetTree {
    private static final int BRANCHES = 26;
    private TrieSetTree[] next;
    private boolean hasElement;
    
    public TrieSetTree() {
        next = new TrieSetTree[BRANCHES];
        hasElement = false;
    }
    
    public void add(String key, int idx) {
        if (key.length() == idx) {
            hasElement = true;
            return;
        }
        
        int index = indexOf(key.charAt(idx));
        if (next[index] == null) {
            next[index] = new TrieSetTree();
        }
        next[index].add(key, idx + 1);
    }
    
    public void remove(String key, int idx) {
        if (key.length() == idx) { 
            hasElement = false;
            return;
        }
        
        int index = indexOf(key.charAt(idx));
        if (next[index] == null) {
            return;
        } else {
            next[index].remove(key, idx + 1);
        }
    }
    
    public boolean contains(String key, int idx) {
        if (key.length() == idx) {
            return hasElement;
        }
        
        int index = indexOf(key.charAt(idx));
        if (next[index] != null) {
            return next[index].contains(key, idx + 1);
        } else {
            return false;
        }
    }
    
    private int indexOf(char c) {
        if ('a' <= c && c <= 'z') { return c - 'a'; }
        if ('A' <= c && c <= 'Z') { return c - 'A'; }
        
        throw new IllegalArgumentException("Invalid : " + c);
    }
}

public class TrieSet {
    private TrieSetTree tree;
    
    public TrieSet() {
        this.tree = new TrieSetTree();
    }
    
    public void add(String key) {
        tree.add(key, 0);
    }
    
    public void remove(String key) {
        tree.remove(key, 0);
    }
    
    public boolean contains(String key) {
        return tree.contains(key, 0);
    }
}



